#ifndef _igc_h_
#define _igc_h_

#include "pch.h"

/*! Fix - describes one GPS fix.
 *
 *  A GPS fix is represented by a timestamp (no day included?),
 *  a longitude, a latitude, the barometric altitude, the
 *  GPS altitude and a flag, if the GPS altitude is valid.
 */
class Fix {
  public: 
    //! Selector for the source of the altitude.
    enum Source { 
	baro,	//!< source from barometer
	gps,	//!< source from gps
    };

    Fix() { }
    Fix(const QString&);    //!< parse the string for the fix

    //! Get the altitude.
    //! \param s source of the altitude (gps or baro)
    //! \return altitude in m
    int altitude(Source s = gps) const { return alt_[s]; }

    QTime time() const { return time_; }    //!< time of the fix
    bool is3d() const { return is3d_; }	    //!< is the gps alt valid?

    //! Get an altitude fix (time+altitude)
    //! \param s source of the altitude (default: gps)
    QPair<QTime, int> altitudeFix(Source s = gps) const { return qMakePair(time_, alt_[s]); }

    QString longitude() const { return gps_.lon; }  //!< longitude of the fix
    QString latitude() const { return gps_.lat; };  //!< latitude of the fix

  private:
    QTime time_;
    bool is3d_;
    struct { QString lon, lat; } gps_;
    int alt_[2];
};

//! List of Fixes.
typedef QList<Fix> Fixes;

/*! Representation of the IGC file.
 *
 *  This class represends the contents of an IGC file.
 *  It reads from a text stream an stores the header data as well as the
 *  fixes. The checksum verification is not implemented currently.
 */
class IGC {
  public:

    IGC();
    IGC(QTextStream&);
    ~IGC();

    //! Read the IGC records.
    //!	\param s a (file) stream containing the records
    void read(QTextStream& s);
    QString device() const { return device_; }	//!< the generator of the IGC file
    QString pilot() const { return pilot_; }	//!< pilots name
    QString glider() const { return glider_; }	//!< type of glider
    QDate date() const { return date_; }	//!< date of the record (start date?)
    
    const Fixes& fixes() const { return fixes_; }	//!< list of fixes

    Fix start() const { return start_; }	//!< the very first fix
    Fix landing() const { return landing_; }	//!< the very last fix
    const QVector<QPointF>& altitude(Fix::Source) const;

  private:
    QString device_, pilot_, glider_;
    QDate date_;
    Fixes fixes_;
    mutable QVector<QPointF>* altitude_[2];

    Fix start_;
    Fix landing_;

};
#endif
